from django.db import models

from ax3_model_extras.storages import get_upload_path


class RedactorFile(models.Model):
    file = models.FileField(upload_to=get_upload_path)

    name = models.CharField(max_length=255)

    is_image = models.BooleanField()

    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ['-id']



from io import BytesIO

from django.contrib.auth.mixins import UserPassesTestMixin
from django.core.files.uploadedfile import SimpleUploadedFile
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from ax3_model_extras.webp import generate_webp

import magic
from PIL import Image

from .models import RedactorFile


@method_decorator(csrf_exempt, name='dispatch')
class RedactorView(UserPassesTestMixin, View):
    def test_func(self):
        return self.request.user.is_authenticated and self.request.user.is_staff

    def get(self, request, *args, **kwargs):
        return JsonResponse([
            {
                'thumb': obj.file.url, 'url': obj.file.url, 'id': obj.id, 'title': obj.name
            } for obj in RedactorFile.objects.filter(is_image=True)[:20]
        ], safe=False)

    def post(self, request, *args, **kwargs):
        count = 0
        images = {}
        for file in request.FILES.getlist('file[]'):
            mime = magic.from_buffer(file.read(), mime=True)
            if mime.startswith('image'):
                output_image = Image.open(file)
                img_format = output_image.format
                if output_image.format == 'JPEG':
                    output_image = output_image.convert('RGB')

                bytes_io = BytesIO()
                output_image.save(bytes_io, format=img_format, optimize=True)

                obj = RedactorFile.objects.create(
                    file=SimpleUploadedFile(file.name, bytes_io.getvalue()),
                    name=file.name,
                    is_image=True,
                )
                generate_webp(image_field=obj.file)
            else:
                obj = RedactorFile.objects.create(file=file, name=file.name, is_image=False)

            images['file-{}'.format(count)] = {'url': obj.file.url, 'name': obj.name, 'id': obj.id}
            count += 1

        return JsonResponse(images, safe=False)



class RedactorMixin:
    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        for field in self.redactor_fields:
            form.base_fields[field].widget.attrs['enable-redactor'] = True
        return form

    class Media:
        css = {
            'all': (
                'vendor/redactor/redactor.min.css',
                'redactor/admin-redactor.css',
            )
        }
        js = (
            'vendor/redactor/redactor.min.js',
            'vendor/redactor/plugins/imagemanager.min.js',
            'vendor/redactor/plugins/video.min.js',
            'redactor/admin-redactor.js',
        )



import setuptools

__VERSION__ = '1.0.3'

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ax3-redactor",
    version=__VERSION__,
    author="Axiacore",
    author_email="info@axiacore.com",
    description="A Django app to add support for redactor",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Axiacore/ax3-redactor",
    packages=setuptools.find_packages(),
    install_requires=['ax3-model-extras==1.1.4', 'python-magic==0.4.15'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
